'COMPANY        :         ALAM JURUTERA PERUNDING SDN. BHD.

'PROGRAM        :         STRUCTURAL ANALYSIS OF PLANE (2D) TRUSSES
'                         BY THE USE OF THE STIFFNESS / EQUILIBRUIM
'                         / DISPLACEMENT METHOD.

'PROGRAMMER     :         TEE HORNG HEAN
'                         B.Eng. (Hons) in Civil Engineering With Construction Management
'                         M.Sc. (Eng) in Structural Engineering Computational Technology

'START DATE     :         10th October 2000

'COMPLETION DATE:         12th October 2000

'ADDITIONAL WORK:         Free Body Diagram
'START DATE     :         17th October 2000
'COMPLETION DATE:         17th October 2000

'COPYRIGHT      :         (C)
'                         This program is a product of Alam Jurutera
'                         Perunding Sdn. Bhd.  Any person who fraudulently
'                         places a copyright on this program, be it compiled,
'                         uncompiled, in another programming language or
'                         any other act which is contrary to the provisions
'                         of Malaysian's Copyright Act shall be subjected to
'                         the detrimental penalties specified therein.

'CAUTION        :          The Assumptions, Concepts and Theories in This
'                         Program are Correct.  THE  ENTIRE RISK  OF  THE
'                         USE  OF THE  RESULTS AND  PERFORMANCE  OF  THIS
'                         SOFTWARE IS  ASSUME ONLY BY YOU!

'TESTING        :         This program has been tested against 3 structures
'                         Only.  Considering the time taken to write this
'                         program, the testing period is very short.
'                         Structure 1 = 3 Joints 3 Members (0,0; 2,2; 2,0)
'                         Structure 2 = 4 Joints 3 Members (-1,0; 1,1; 2,1; 0,0)
'                         Structure 3 = 23 Joints 50 Members
'                         Results (OK)

'REFERENCES     :        
'                         2. Reddy, C.S. 1981, "Basic Structural Analysis", TaTa Mc Graw-Hill.

REM CLEAR SCREEN
REM ------------
CLS

REM INPUT PHYSICAL PROPERTIES
REM -------------------------
   'Number of Joints and Number of Members
   PRINT "Number of Joints :";
   INPUT "", NoOfJoints
   PRINT
   PRINT "Number of Members :";
   INPUT "", NoOfmembers
   PRINT

   'Joint Coordinates Input
   DIM XCoord(NoOfJoints), YCoord(NoOfJoints)
   FOR I = 1 TO NoOfJoints
	 PRINT "Coordinates of Joint "; I; " (X,Y) :";
	 INPUT "", XCoord(I), YCoord(I)
   NEXT I
   PRINT
  
   PRINT "Assuming you have a member connected by Joint F and G and you assumed that the"
   PRINT "local x axes is pointing from F to G, then your Joint i is F and Joint j is G"
   PRINT "and vice versa."
   PRINT

   'Member Connectivity and Properties such as E,A
   DIM JN(NoOfmembers, 2), Area(NoOfmembers), YM(NoOfmembers)
   FOR I = 1 TO NoOfmembers
	 PRINT "Joint Ends of Member "; I; " (Joint i,Joint j):";
	 INPUT "", JN(I, 1), JN(I, 2)
	 PRINT "Young's Modulus of Member "; I; " :";
	 INPUT "", YM(I)
	 PRINT "Cross Sectional Area of Member "; I; " :";
	 INPUT "", Area(I)
   NEXT I
   PRINT

'-----------------------ADDITIONAL--------------------------------
'THIS IS A FREE BODY DIAGRAM ROUTINE
'THIS CAN BE MODIFIED FOR AUTOCAD PLOTTING

'Search for max and min values of coordinates
	MaxX = XCoord(1)
	MaxY = YCoord(1)
	MinX = XCoord(1)
	MinY = YCoord(1)
	FOR I = 1 TO NoOfJoints
		IF XCoord(I) >= MaxX THEN MaxX = XCoord(I)
		IF YCoord(I) >= MaxY THEN MaxY = YCoord(I)
		IF XCoord(I) <= MinX THEN MinX = XCoord(I)
		IF YCoord(I) <= MinY THEN MinY = YCoord(I)
	NEXT I
	TempDiffX = MaxX - MinX
	TempDiffY = MaxY - MinY

	'Scaling
	ScreenWidth = 600
	ScreenDepth = 400
	XScale = ScreenWidth / TempDiffX
	YScale = ScreenDepth / TempDiffY

	SCREEN 12
	CLS

	'Start Plotting Joints
	Boundary = 10
	FOR I = 1 TO NoOfJoints
		TempX = Boundary + XCoord(I) * XScale
		TempY = Boundary + ScreenDepth - YCoord(I) * YScale
		CIRCLE (TempX, TempY), 2, 15
		A$ = CHR$(122) + CHR$(133)
		PAINT (TempX, TempY), A$, 15
	NEXT I
    
	'Start Drawing Members
	FOR I = 1 TO NoOfmembers
		A = JN(I, 1)
		B = JN(I, 2)
		TempX1 = Boundary + XCoord(A) * XScale
		TempY1 = Boundary + ScreenDepth - YCoord(A) * YScale
		TempX2 = Boundary + XCoord(B) * XScale
		TempY2 = Boundary + ScreenDepth - YCoord(B) * YScale
		LINE (TempX1, TempY1)-(TempX2, TempY2), 15
	NEXT I
	LOCATE 19: PRINT "Hit A Key To Continue": TEE$ = INPUT$(1)
	SCREEN 0
	CLS
'-----------------------ADDITIONAL--------------------------------


   'Degree of Freedom = Size of Structure Stiffness Matrix (DOF,DOF)
   DOF = NoOfJoints * 2
   'Global Structure Stiffness Matrix, External Force, Local Member Stiffness
   DIM GlStK(DOF, DOF), OriginalK(DOF, DOF), ExtLoad(DOF)
   DIM LoMemK(NoOfmembers, 2, 2), FIXX(NoOfJoints), FIXY(NoOfJoints)
   DIM Disp(DOF)

   'Loads Applied on the Joints
   FOR I = 1 TO NoOfJoints
	 PRINT "Loading on Joint "; I; " (Fx,Fy) :";
	 INPUT "", ExtLoad(I * 2 - 1), ExtLoad(I * 2)
   NEXT I
						
REM ASSEMBLY OF STRUCTURE STIFFNESS MATRIX
REM --------------------------------------
   'Initially Evaluate The Lengths, Cos and Sine, and Member Stiffness Matrix
   FOR I = 1 TO NoOfmembers
	 Jointi = JN(I, 1)
	 Jointj = JN(I, 2)
	 DIFFX = XCoord(Jointj) - XCoord(Jointi)
	 DIFFY = YCoord(Jointj) - YCoord(Jointi)
	 MemberLength = SQR(DIFFX ^ 2 + DIFFY ^ 2)
	 cosB = DIFFX / MemberLength
	 sinB = DIFFY / MemberLength
	 STIFF = YM(I) * Area(I) / MemberLength
	 LoMemK(I, 1, 1) = STIFF * cosB ^ 2
	 LoMemK(I, 1, 2) = STIFF * sinB * cosB
	 LoMemK(I, 2, 1) = STIFF * cosB * sinB
	 LoMemK(I, 2, 2) = STIFF * sinB ^ 2
   NEXT I


	'Struct. Stiffness Matrix - DIAGONALS
	'The Loop N is to Cycle Thru the number of Joints. But in actual fact,
	'it is actually CONSIDERING the ROWS in the Structural Matix. You see,
	'The rows of the Stuct. Matrix is porpotional to the NoOfJoints
	'J is to cycle thru members and K is to cycle thru the Joint Endings
	'Thus, in layman;s terms, "IS Joint N connected to Member J's Joint End
	'Number K?"
	FOR N = 1 TO NoOfJoints
		FOR J = 1 TO NoOfmembers
			FOR K = 1 TO 2
				IF JN(J, K) = N THEN
				'PRINT N, J, K
				
				 'The Row of Global Stiffness Matrix
				 Row = N * 2 - 1
				 'COL = ROW cause we are inputing matrix diagonals first
				 COL = Row
				 GlStK(Row, COL) = GlStK(Row, COL) + LoMemK(J, 1, 1)
				 GlStK(Row, COL + 1) = GlStK(Row, COL + 1) + LoMemK(J, 1, 2)
				 GlStK(Row + 1, COL) = GlStK(Row + 1, COL) + LoMemK(J, 2, 1)
				 GlStK(Row + 1, COL + 1) = GlStK(Row + 1, COL + 1) + LoMemK(J, 2, 2)
				END IF
			NEXT K
		NEXT J
	NEXT N


   'Struct. Stiffness Matrix - OFF - DIAGONALS
   'The Loops N and J are to Cycle Thru Members. In Layman's Terms, you ask,
   '"IS MEMBER N CONNECTED TO MEMBER J?". Loop K and L are Joint Ends, THUS,
   '"IS MEMBER N's Joint End K CONNECTED TO MEMBER J's Joint END L?".
   FOR N = 1 TO NoOfmembers
	 FOR J = 1 TO NoOfmembers
	    FOR K = 1 TO 2
		  FOR L = 1 TO 2
			'PRINT N, K, J, L
			   'You have found that Member N and J has same Joints
			   IF JN(N, K) = JN(J, L) THEN
				 'The Row of Global Stiffness Matrix
				 Row = JN(N, K) * 2 - 1
				 'COL = ROW cause we are inputing matrix diagonals first
				 COL = Row
				 'GlStK(ROW, COL) = GlStK(ROW, COL) + LoMemK(N, 1, 1)
				 'GlStK(ROW, COL + 1) = GlStK(ROW, COL + 1) + LoMemK(N, 1, 2)
				 'GlStK(ROW + 1, COL) = GlStK(ROW + 1, COL) + LoMemK(N, 2, 1)
				 'GlStK(ROW + 1, COL + 1) = GlStK(ROW + 1, COL + 1) + LoMemK(N, 2, 2)
		 
				 'Now You Start Filling The Rest Of Columns i.e. off Diagonals
				 'but at the same Row
				
				 'MEMBER N
				 'Initially Find Opposite End of Member N
				 TempOpp = K
				 IF TempOpp = 1 THEN
				    TempOpp = 2
				    ELSE
				    TempOpp = 1
				 END IF
				 COL = JN(N, TempOpp) * 2 - 1
				 GlStK(Row, COL) = -LoMemK(N, 1, 1)
				 GlStK(Row, COL + 1) = -LoMemK(N, 1, 2)
				 GlStK(Row + 1, COL) = -LoMemK(N, 2, 1)
				 GlStK(Row + 1, COL + 1) = -LoMemK(N, 2, 2)
				
				 'MEMBER J
				 'Initially Find Opposite End of Member J
				 TempOpp = L
				 IF TempOpp = 1 THEN
				    TempOpp = 2
				    ELSE
				    TempOpp = 1
				 END IF
				 COL = JN(J, TempOpp) * 2 - 1
				 GlStK(Row, COL) = -LoMemK(J, 1, 1)
				 GlStK(Row, COL + 1) = -LoMemK(J, 1, 2)
				 GlStK(Row + 1, COL) = -LoMemK(J, 2, 1)
				 GlStK(Row + 1, COL + 1) = -LoMemK(J, 2, 2)
			  
			   END IF
		  NEXT L
	    NEXT K
	 NEXT J
   NEXT N

PRINT

'OUTPUT OF STRUCTURAL STIFFNESS MATRIX
PRINT "Now Outputting Structural Stiffness Matrix"
PRINT
FOR I = 1 TO DOF
	FOR J = 1 TO DOF
		PRINT USING "+##.###"; GlStK(I, J);
		OriginalK(I, J) = GlStK(I, J)'make Original Read Only - for use in reactions
	NEXT J
	PRINT
NEXT I
PRINT

REM BOUNDARY CONDITIONS
REM -------------------
  PRINT "1 = Fixed ; 0 = Free"
  FOR I = 1 TO NoOfJoints
	  PRINT "Conditions for Joint "; I; " (X,Y):";
	  INPUT "", FX, FY
	  Row = (I * 2) - 1
	  COL = Row
	  IF FX = 1 THEN
		GlStK(Row, COL) = 10 ^ 30
		'GlStK(ROW + 1, COL) = 10 ^ 30
		ExtLoad(Row) = 0
		FIXX(I) = 1
	  END IF
	  IF FY = 1 THEN
		GlStK(Row + 1, COL + 1) = 10 ^ 30
		'GlStK(ROW, COL + 1) = 10 ^ 30
		ExtLoad(Row + 1) = 0
		FIXY(I) = 1
	  END IF
   NEXT I

'GAUSS ELIMINATION
'ALGORITHM DOWNLOADED FROM INTERNET
N = DOF
'FORWARD ELEMINATION
ell = N - 1
FOR i1 = 1 TO ell
	elx = i1 + 1
	pij = i1
	pibig = ABS(GlStK(i1, i1))
	FOR I = elx TO N
		am = ABS(GlStK(I, i1))
		IF (am > pibig) THEN
			pibig = am
			pij = I
		END IF
	NEXT
	IF (pij <> i1) THEN
		FOR J = i1 TO N
			pidum = GlStK(pij, J)
			GlStK(pij, J) = GlStK(i1, J)
			GlStK(i1, J) = pidum
		NEXT J
	pitemp = ExtLoad(pij)
	ExtLoad(pij) = ExtLoad(i1)
	ExtLoad(i1) = pitemp
	END IF
	FOR i2 = elx TO N
		FOR J = elx TO N
			GlStK(i2, J) = GlStK(i2, J) - GlStK(i2, i1) / GlStK(i1, i1) * GlStK(i1, J)
		NEXT
		ExtLoad(i2) = ExtLoad(i2) - GlStK(i2, i1) / GlStK(i1, i1) * ExtLoad(i1)
	NEXT
NEXT i1

'BACK SUBSTITUTE
sul = N - 1
Disp(N) = ExtLoad(N) / GlStK(N, N)

FOR ix = 1 TO sul
	susum = 0
	sui = N - ix
	suj = sui + 1
	FOR jx = suj TO N
		susum = susum + GlStK(sui, jx) * Disp(jx)
	NEXT
	Disp(sui) = (ExtLoad(sui) - susum) / GlStK(sui, sui)
NEXT
PRINT

'OUTPUT OF DISPLACEMENTS
PRINT "DISPACEMENTS"
FOR I = 1 TO NoOfJoints
	PRINT "Joint "; I; "X ="; USING "+##.###"; Disp(I * 2 - 1);
	PRINT , "Y ="; USING "+##.###"; Disp(I * 2)
	TEE$ = INPUT$(1)
NEXT I
PRINT

'OUTPUT OF MEMBER FORCES
'We have displacements in Global Terms, so,
'where Fj is joint end j - Remember Hooke's Law
'Fj' = -k [ cos B  sin B ] [u1  u2]  + K  [ cos B  sin B ] [u1  u2]
PRINT "MEMBER FORCES"
   FOR I = 1 TO NoOfmembers
	 Jointi = JN(I, 1)
	 Jointj = JN(I, 2)
	 DIFFX = XCoord(Jointj) - XCoord(Jointi)
	 DIFFY = YCoord(Jointj) - YCoord(Jointi)
	 MemberLength = SQR(DIFFX ^ 2 + DIFFY ^ 2)
	 cosB = DIFFX / MemberLength
	 sinB = DIFFY / MemberLength
	 STIFF = YM(I) * Area(I) / MemberLength
	 u1 = Disp(Jointi * 2 - 1)
	 u2 = Disp(Jointj * 2 - 1)
	 v1 = Disp(Jointi * 2)
	 v2 = Disp(Jointj * 2)
	 Force = -STIFF * (cosB * u1 + sinB * v1) + STIFF * (cosB * u2 + sinB * v2)
	 PRINT "Force of Member "; I; "="; USING "+##.###"; Force
	 TEE$ = INPUT$(1)
   NEXT I
PRINT

'REACTIONS
'initialisation
ReactionX = 0
ReactionY = 0
FOR I = 1 TO NoOfJoints
	'this means if The joint is fixed then do it
	'when it's (joint) is free, there is no reaction
	IF FIXX(I) = 1 THEN
		Row = (I - 1) * 2 + 1
		FOR J = 1 TO DOF
			ReactionX = ReactionX + OriginalK(Row, J) * Disp(J)
		NEXT J
	END IF

	IF FIXY(I) = 1 THEN
		Row = (I - 1) * 2 + 2
		FOR J = 1 TO DOF
			ReactionY = ReactionY + OriginalK(Row, J) * Disp(J)
		NEXT J
	END IF
	PRINT "Reactions of Joint "; I; "X ="; USING "+##.####"; ReactionX;
	PRINT , " Y ="; USING "+##.###"; ReactionY
	ReactionX = 0
	ReactionY = 0
	TEE$ = INPUT$(1)
NEXT I


'-----------------------ADDITIONAL--------------------------------
'THIS IS A FREE BODY DIAGRAM ROUTINE
'THIS CAN BE MODIFIED FOR AUTOCAD PLOTTING

'Search for max and min values of coordinates
	MaxX = XCoord(1)
	MaxY = YCoord(1)
	MinX = XCoord(1)
	MinY = YCoord(1)
	MaxDisp = Diff(1)

	FOR I = 1 TO NoOfJoints
		IF FIXX(I) = 1 THEN Disp(I * 2 - 1) = 0
		IF FIXY(I) = 1 THEN Disp(I * 2 - 0) = 0
	NEXT I

	FOR I = 1 TO NoOfJoints
		IF XCoord(I) >= MaxX THEN MaxX = XCoord(I)
		IF YCoord(I) >= MaxY THEN MaxY = YCoord(I)
		IF XCoord(I) <= MinX THEN MinX = XCoord(I)
		IF YCoord(I) <= MinY THEN MinY = YCoord(I)
		IF MaxDispX >= Disp(I * 2 - 1) THEN MaxDispX = Disp(I * 2 - 1)
		IF MinDispX <= Disp(I * 2 - 1) THEN MinDispX = Disp(I * 2 - 1)
		IF MaxDispY >= Disp(I * 2) THEN MaxDispY = Disp(I * 2)
		IF MinDispY <= Disp(I * 2) THEN MinDispY = Disp(I * 2)

	NEXT I
	TempDiffX = ABS(MaxX - MinX)
	TempDiffY = ABS(MaxY - MinY)
	TempDispX = ABS(MaxDispX - MinDispX)
	TempDispY = ABS(MaxDispY - MinDispY)


	'Scaling
	ScreenWidth = 400
	ScreenDepth = 300
	XScale = ScreenWidth / TempDiffX
	YScale = ScreenDepth / TempDiffY
	IF TempDispX = 0 THEN TempDispX = 1
	IF TempDispY = 0 THEN TempDispY = 1
	IF TempDispX > TempDispY THEN
		TempX = TempDispX / TempDispY
		ELSE
		TempX = TempDispY / TempDispX
	END IF
	DispXScale = 50 * TempX / TempDispX
	DispYScale = 50 * TempX / TempDispY

	SCREEN 12
	CLS
	PRINT "+ or - animation speed control"
	'Start Animating Drawing Deflected Members
	ANIMATE = 1
	ANIMATEFAC = 1
	Frames = 20
	INTERVAL = 5000

	WHILE ANIMATE < 21 OR ANIMATE > 0

		FOR I = 1 TO NoOfmembers
		
			A = JN(I, 1)
			B = JN(I, 2)

			TempX1 = Boundary + (XCoord(A) * XScale + (Disp(A * 2 - 1) * DispXScale) / Frames * ANIMATE)
			TempY1 = Boundary + ScreenDepth - (YCoord(A) * YScale + (Disp(A * 2) * DispYScale) / Frames * ANIMATE)
			TempX2 = Boundary + (XCoord(B) * XScale + (Disp(B * 2 - 1) * DispXScale) / Frames * ANIMATE)
			TempY2 = Boundary + ScreenDepth - (YCoord(B) * YScale + (Disp(B * 2) * DispYScale) / Frames * ANIMATE)

			LINE (TempX1, TempY1)-(TempX2, TempY2), 14
		NEXT I
	    
		FOR TEE = 1 TO INTERVAL
			'PAUSE
			A$ = INKEY$
			IF A$ = "+" OR A$ = "=" THEN INTERVAL = INTERVAL + 100
			IF A$ = "-" OR A$ = "_" THEN INTERVAL = INTERVAL - 100
			IF INTERVAL <= 500 THEN INTERVAL = 500
		NEXT TEE
	    
		FOR I = 1 TO NoOfmembers
	    
			A = JN(I, 1)
			B = JN(I, 2)
			TempX1 = Boundary + (XCoord(A) * XScale + (Disp(A * 2 - 1) * DispXScale) / Frames * ANIMATE)
			TempY1 = Boundary + ScreenDepth - (YCoord(A) * YScale + (Disp(A * 2) * DispYScale) / Frames * ANIMATE)
			TempX2 = Boundary + (XCoord(B) * XScale + (Disp(B * 2 - 1) * DispXScale) / Frames * ANIMATE)
			TempY2 = Boundary + ScreenDepth - (YCoord(B) * YScale + (Disp(B * 2) * DispYScale) / Frames * ANIMATE)


			LINE (TempX1, TempY1)-(TempX2, TempY2), 0

		    
			
	    
	    
		NEXT I
			
		   
			IF ANIMATE >= 21 OR ANIMATE <= 0 THEN
				ANIMATEFAC = ANIMATEFAC * -1
			END IF
			IF ANIMATE >= 21 THEN ANIMATE = 20
			IF ANIMATE <= 0 THEN ANIMATE = 1
			ANIMATE = ANIMATE + ANIMATEFAC
			
	'Start Plotting Joints
	Boundary = 50 + MaxDisp
	FOR I = 1 TO NoOfJoints
		TempX = Boundary + XCoord(I) * XScale
		TempY = Boundary + ScreenDepth - YCoord(I) * YScale
		CIRCLE (TempX, TempY), 2, 15
		A$ = CHR$(122) + CHR$(133)
		PAINT (TempX, TempY), A$, 15
	NEXT I

	'Start Plotting Deflected Joints
	Boundary = 50
	FOR I = 1 TO NoOfJoints
		TempX = Boundary + (XCoord(I) * XScale + Disp(I * 2 - 1) * DispXScale)
		TempY = Boundary + ScreenDepth - (YCoord(I) * YScale + Disp(I * 2) * DispYScale)
		CIRCLE (TempX, TempY), 2, 13
		A$ = CHR$(122) + CHR$(133)
		PAINT (TempX, TempY), A$, 13
	NEXT I

  
	'Start Drawing Members
	FOR I = 1 TO NoOfmembers
		A = JN(I, 1)
		B = JN(I, 2)
			    
		TempX1 = Boundary + XCoord(A) * XScale
		TempY1 = Boundary + ScreenDepth - YCoord(A) * YScale
		TempX2 = Boundary + XCoord(B) * XScale
		TempY2 = Boundary + ScreenDepth - YCoord(B) * YScale
		LINE (TempX1, TempY1)-(TempX2, TempY2), 15
			    
	NEXT I



	WEND
	
	 


'-----------------------ADDITIONAL--------------------------------

